@@ -27,7 +27,6 @@ import gnomeapplet
 import gobject
 import gtk
 from gtk import gdk
-import Numeric
 
 import os
 from xml.sax import saxutils
@@ -41,6 +40,8 @@ STOCK_ICON = "music-applet"
 STOCK_SET_STAR = "music-applet-set-star"
 STOCK_UNSET_STAR = "music-applet-unset-star"
 
+USE_GTK_TOOLTIP = (gtk.check_version (2,12,0) == None)
+
 
 class MusicApplet:
     """
@@ -60,7 +61,6 @@ class MusicApplet:
         gtk.window_set_default_icon_from_file (os.path.join (musicapplet.defs.PKG_DATA_DIR, "%s.png" % STOCK_ICON))
 
         self.__applet = applet
-        self.__tooltips = gtk.Tooltips ()
         self.__all = None
 
         self.__prefs_dialog = None
@@ -73,7 +73,6 @@ class MusicApplet:
         applet.add_preferences ("/schemas/apps/music-applet/prefs")
         self.__conf = musicapplet.conf.Conf (applet)
 
-        self.__song_tip = SongTooltip ()
         self.__manager = musicapplet.manager.Manager (self.__conf, [
             os.path.join (musicapplet.defs.PYTHON_DIR, "musicapplet/plugins"),
             os.path.expanduser ("~/.gnome2/music-applet/plugins"),
@@ -84,10 +83,16 @@ class MusicApplet:
         except ImportError:
             self.__notify = None
 
+	# Tooltips
+
+        self.__song_tip_guts = SongTooltipGuts ()
+        if not USE_GTK_TOOLTIP:
+            self.__tooltips = gtk.Tooltips ()
+
         # Rating
 
         self.__rating = Rating ()
-        self.__tooltips.set_tip (self.__rating, _("Rate current song"))
+	self.__set_widget_tooltip (self.__rating, _("Rate current song"))
         self.__rating.connect ("notify::rating", self.__rate_song_cb)
         full_key = self.__conf.resolve_key ("show_rating")
         self.__conf.client.notify_add (full_key,
@@ -99,7 +104,12 @@ class MusicApplet:
         self.__time = gtk.Label ("--:--")
         self.__time.show ()
         self.__event_box = gtk.EventBox ()
-        self.__song_tip.set_tip (self.__event_box, "")
+        if USE_GTK_TOOLTIP:
+            self.__event_box.props.has_tooltip = True
+            self.__event_box.connect ("query-tooltip", self.__query_tooltip_cb)
+        else:
+            song_tip = SongTooltip (self.__song_tip_guts)
+            song_tip.set_tip (self.__event_box, "")
         self.__event_box.add (self.__time)
         full_key = self.__conf.resolve_key ("show_time")
         self.__conf.client.notify_add (full_key,
@@ -110,17 +120,17 @@ class MusicApplet:
 
         self.__previous = musicapplet.widgets.FancyButton ()
         self.__previous.set_stock_id (gtk.STOCK_MEDIA_PREVIOUS)
-        self.__tooltips.set_tip (self.__previous, _("Previous song"))
+	self.__set_widget_tooltip (self.__previous, _("Previous song"))
         self.__previous.connect ("clicked", self.__previous_clicked_cb)
 
         self.__play = musicapplet.widgets.FancyButton ()
         self.__play.set_stock_id (gtk.STOCK_MEDIA_PLAY)
-        self.__tooltips.set_tip (self.__play, _("Start playing"))
+	self.__set_widget_tooltip (self.__play, _("Start playing"))
         self.__play.connect ("clicked", self.__play_clicked_cb)
 
         self.__next = musicapplet.widgets.FancyButton ()
         self.__next.set_stock_id (gtk.STOCK_MEDIA_NEXT)
-        self.__tooltips.set_tip (self.__next, _("Next song"))
+	self.__set_widget_tooltip (self.__next, _("Next song"))
         self.__next.connect ("clicked", self.__next_clicked_cb)
 
         full_key = self.__conf.resolve_key ("show_controls")
@@ -157,6 +167,19 @@ class MusicApplet:
         self.__manager.connect ("notify::preferred",
                                 lambda manager, pspec: self.__set_preferred (manager.preferred))
 
+    def __set_widget_tooltip (self, widget, text):
+        if USE_GTK_TOOLTIP:
+	    widget.set_tooltip_text (text)
+	else:
+	    self.__tooltips.set_tip (widget, text)
+
+    def __query_tooltip_cb (self, widget, x, y, kbd, tooltip):
+        # Yes, a new Guts each time, since GTK destroys it along with the tooltip itself.
+        guts = SongTooltipGuts ()
+        guts.show_all ()
+        guts.set_plugin (self.__plugin)
+        tooltip.set_custom (guts)
+        return True
 
     def __set_plugin (self, plugin):
         for handler in self.__plugin_handlers:
@@ -180,7 +203,7 @@ class MusicApplet:
             self.__plugin_handlers = []
             self.__rating.interactive = False
 
-        self.__song_tip.set_plugin (plugin)
+        self.__song_tip_guts.set_plugin (plugin)
         if self.__notify is not None:
             self.__notify.set_plugin (plugin)
         self.__update_rating_visibility ()
@@ -199,10 +222,10 @@ class MusicApplet:
                 self.__launch.set_icon_name (preferred.icon_name)
             else:
                 self.__launch.set_stock_id ("music-applet")
-            self.__tooltips.set_tip (self.__launch, _("Launch %s") % preferred.player_name)
+	    self.__set_widget_tooltip (self.__launch, _("Launch %s") % preferred.player_name)
         else:
             self.__launch.set_stock_id ("music-applet")
-            self.__tooltips.set_tip (self.__launch, _("Launch preferred player"))
+	    self.__set_widget_tooltip (self.__launch, _("Launch preferred player"))
 
 
     def __update_rating_visibility (self, conf_value=None):
@@ -333,10 +356,10 @@ class MusicApplet:
     def __playing_changed_cb (self, plugin, pspec):
         if plugin.playing:
             self.__play.set_stock_id (gtk.STOCK_MEDIA_PAUSE)
-            self.__tooltips.set_tip (self.__play, _("Pause playback"))
+	    self.__set_widget_tooltip (self.__play, _("Pause playback"))
         else:
             self.__play.set_stock_id (gtk.STOCK_MEDIA_PLAY)
-            self.__tooltips.set_tip (self.__play, _("Start playing"))
+	    self.__set_widget_tooltip (self.__play, _("Start playing"))
 
 
     def __elapsed_changed_cb (self, plugin, pspec):
@@ -357,14 +380,14 @@ class MusicApplet:
     def __rating_changed_cb (self, plugin, pspec):
         self.__rating.rating = plugin.rating
         if "rate_song" not in dir (plugin):
-            self.__tooltips.set_tip (self.__rating, _("%s does not support rating songs") %
+            self.__set_widget_tooltip (self.__rating, _("%s does not support rating songs") %
                                                     self.__plugin.player_name)
             self.__rating.interactive = False
         elif plugin.rating < 0:
-            self.__tooltips.set_tip (self.__rating, _("No song to rate"))
+            self.__set_widget_tooltip (self.__rating, _("No song to rate"))
             self.__rating.interactive = False
         else:
-            self.__tooltips.set_tip (self.__rating, _("Rate current song"))
+            self.__set_widget_tooltip (self.__rating, _("Rate current song"))
             self.__rating.interactive = True
 
 
@@ -424,10 +447,23 @@ class MusicApplet:
         self.__about_dialog = None
 
 
-
 class SongTooltip (musicapplet.widgets.FancyTooltips):
     """
-    A tooltip that displays information about the current song.
+    Old-style tooltip using a custom GtkTooltips subclass.
+    """
+    def __init__ (self, guts):
+    	musicapplet.widgets.FancyTooltips.__init__ (self)
+        self.__guts = guts
+	self.__guts.show_all ()
+	self.set_content (self.__guts)
+
+    def set_plugin (self, plugin):
+    	self.__guts.set_plugin (plugin)
+
+
+class SongTooltipGuts (gtk.HBox):
+    """
+    Contents of a tooltip that displays information about the current song.
     """
 
     ##################################################################
@@ -437,7 +473,7 @@ class SongTooltip (musicapplet.widgets.FancyTooltips):
     ##################################################################
 
     def __init__ (self):
-        musicapplet.widgets.FancyTooltips.__init__ (self)
+	gtk.HBox.__init__(self, False, 6)
         self.__handlers = []
 
         self.__title = gtk.Label ()
@@ -451,15 +487,8 @@ class SongTooltip (musicapplet.widgets.FancyTooltips):
             label.set_alignment (0.0, 0.5)
             vbox.add (label)
 
-        hbox = gtk.HBox ()
-        hbox.set_spacing (6)
-        hbox.add (self.__art)
-        hbox.add (vbox)
-
-        hbox.show_all ()
-        self.set_content (hbox)
-
-        self.set_plugin (None)
+	self.add (self.__art)
+	self.add (vbox)
 
 
     def set_plugin (self, plugin):
@@ -735,7 +764,7 @@ class Rating (gtk.EventBox):
         for row in pixbuf.get_pixels_array ():
             for pixel in row:
                 # Yay API changes!
-                if Numeric.__version__ >= 24.0:
+                if str (type (pixel[0])) == "<type 'array'>":
                     pixel[0][0] *= color.red
                     pixel[1][0] *= color.green
                     pixel[2][0] *= color.blue
@@ -805,7 +834,7 @@ class NotifyPopup (gobject.GObject):
             if self.__plugin.album is not None:
                 body.append (_("<i>from</i> %s") % saxutils.escape (self.__plugin.album))
 
-            self.__notify = pynotify.Notification (self.__plugin.title, "\n".join (body), None, self.__applet)
+            self.__notify = pynotify.Notification (saxutils.escape (self.__plugin.title), "\n".join (body), None, self.__applet)
             self.__notify.set_urgency (pynotify.URGENCY_LOW)
             if self.__plugin.art is not None:
                 self.__notify.set_icon_from_pixbuf (scale_pixbuf (self.__plugin.art))
